@@ -65,7 +65,7 @@ def solve(form, display=True, export=False):
                                     [mosek.variabletype.type_int]
                                     * len(ind_bin))
 
-            task.putcslice(0, numvar, form.obj[0])
+            task.putcslice(0, numvar, form.obj.flatten())
             task.putobjsense(mosek.objsense.minimize)
 
             coo = coo_matrix(form.linear)
@@ -95,7 +95,6 @@ def solve(form, display=True, export=False):
             solsta = None
             for stype in [soltype.bas, soltype.itr, soltype.itg]:
                 try:
-                    # prosta = task.getprosta(mosek.soltype.itr)
                     solsta = task.getsolsta(stype)
                     if display:
                         print('Solution status: {0}'.format(solsta.__repr__()))
