@@ -34,6 +34,15 @@ class Model:
 
         self.name = name
 
+    def reset(self):
+
+        self.all_constr = []
+        self.pupdate = True
+        self.dupdate = True
+        self.primal = None
+        self.dual = None
+        self.rc_model.reset()
+
     def dvar(self, shape=(1,), vtype='C', name=None, aux=False):
         """
         Returns an array of decision variables with the given shape
@@ -52,7 +61,7 @@ class Model:
 
         Returns
         -------
-        new_var : road.lp.Vars
+        new_var : rsome.lp.Vars
             An array of new decision variables
         """
 
@@ -73,7 +82,7 @@ class Model:
 
         Returns
         -------
-        new_var : road.lp.Vars
+        new_var : rsome.lp.Vars
             An array of new random variables
         """
 
@@ -82,6 +91,23 @@ class Model:
 
     def ldr(self, shape=(1,), name=None):
 
+        """
+        Returns an array with the given shape of linear decision rule
+        variables.
+
+        Parameters
+        ----------
+        shape : int or tuple
+            Shape of the variable array.
+        name : str
+            Name of the variable array
+
+        Returns
+        -------
+        new_var : rsome.ro.DecRule
+            An array of new linear decision rule variables
+        """
+
         new_ldr = DecRule(self, shape, name)
         return new_ldr
 
@@ -171,7 +197,10 @@ class Model:
                 if (constr.dec_model is not self.rc_model) or \
                         (constr.rand_model is not self.sup_model):
                     raise ValueError('Models mismatch.')
-                if constr.sense == 0:
+                sense = (constr.sense[0] if isinstance(constr.sense,
+                                                       np.ndarray)
+                         else constr.sense)
+                if sense == 0:
                     self.all_constr.append(constr)
                 else:
                     left = RoAffine(constr.raffine, constr.affine,
@@ -293,14 +322,18 @@ class DecRule:
         if self.depend is None:
             self.depend = np.zeros((self.size,
                                     self.model.sup_model.vars[-1].last),
-                                   dtype=np.int8)
+                                   dtype=int)
 
         indices = rvar.get_ind()
         if ldr_indices is None:
-            ldr_indices = np.arange(self.depend.shape[0], dtype=np.int8)
+            ldr_indices = np.arange(self.depend.shape[0], dtype=int)
         ldr_indices = ldr_indices.reshape((ldr_indices.size, 1))
 
-        if self.depend[ldr_indices, indices].any():
+        row_ind = (ldr_indices *
+                   np.ones(indices.shape, dtype=int)).flatten()
+        col_ind = (np.ones(ldr_indices.shape, dtype=int) * indices).flatten()
+
+        if self.depend[row_ind, col_ind].any():
             raise SyntaxError('Redefinition of adaptation is not allowed.')
 
         self.depend[ldr_indices, indices] = 1
@@ -310,8 +343,8 @@ class DecRule:
         if self.roaffine is not None:
             return self.roaffine
         else:
-            num_ones = self.depend.sum()
-            if num_ones:
+            if self.depend is not None:
+                num_ones = self.depend.sum()
                 var_coeff = self.model.dvar(num_ones)
                 self.var_coeff = var_coeff
                 row_ind = np.where(self.depend.flatten() == 1)[0]
