@@ -80,7 +80,11 @@ class Model(LPModel):
                     aux_right = self.dvar(constr.affine_out.shape, aux=True)
                     self.aux_constr.append(constr.affine_in - aux_left == 0)
                     self.aux_constr.append(constr.affine_out + aux_right == 0)
-                    self.aux_bounds.append(aux_right >= 0)
+                    bounds = (aux_right >= 0)
+                    if isinstance(bounds, Bounds):
+                        self.aux_bounds.append(bounds)
+                    else:
+                        self.aux_constr.append(bounds)
                     qmat.append([aux_right.first] +
                                 list([aux_left.first + index
                                       for index in range(aux_left.size)]))
@@ -93,7 +97,11 @@ class Model(LPModel):
                     self.aux_constr.append(aux2 - constr.affine_in == 0)
                     self.aux_constr.append(aux3 - 0.5 * (1-constr.affine_out)
                                            == 0)
-                    self.aux_bounds.append(aux3 >= 0)
+                    bounds = (aux3 >= 0)
+                    if isinstance(bounds, Bounds):
+                        self.aux_bounds.append(bounds)
+                    else:
+                        self.aux_constr.append(bounds)
                     for i in range(constr.affine_in.size):
                         qmat.append([aux3.first + i] +
                                     [aux1.first + i, aux2.first + i])
@@ -106,7 +114,11 @@ class Model(LPModel):
                     self.aux_constr.append(aux2 - constr.affine_in == 0)
                     self.aux_constr.append(aux3 - 0.5 * (1-constr.affine_out)
                                            == 0)
-                    self.aux_bounds.append(aux3 >= 0)
+                    bounds = (aux3 >= 0)
+                    if isinstance(bounds, Bounds):
+                        self.aux_bounds.append(bounds)
+                    else:
+                        self.aux_constr.append(bounds)
                     qmat.append([aux3.first] + [aux1.first] +
                                 list(aux2.first + np.arange(aux2.size)))
 
@@ -127,13 +139,15 @@ class Model(LPModel):
             if self.dual is not None and not self.dupdate:
                 return self.dual
 
-            primal = self.do_math()
+            primal = self.do_math(obj=obj)
 
-            dual_lp = super().do_math(primal=False, refresh=False)
+            dual_lp = super().do_math(primal=False, refresh=False, obj=obj)
             if len(primal.qmat) == 0:
-                return SOCProg(dual_lp.linear, dual_lp.const, dual_lp.sense,
-                               dual_lp.vtype, dual_lp.ub, dual_lp.lb,
-                               [], dual_lp.obj)
+                formula = SOCProg(dual_lp.linear, dual_lp.const, dual_lp.sense,
+                                  dual_lp.vtype, dual_lp.ub, dual_lp.lb,
+                                  [], dual_lp.obj)
+                self.dual = formula
+                return formula
 
             eye_indices = [item for inner in primal.qmat for item in inner]
             lin_indices = [ind for ind in range(primal.linear.shape[1])
